function [bestX, bestFx] = mc2(fx, XLow, XHi, InitStep, FinStep, MaxMCiters, MaxHCiters)
% Pure random search in trust region range followed by hill climbing
% INPUT
% =====
% fx - handle of optimized function.
% XLow - array of lower limits of trust region.
% XHi - array of upper limits of trust region.
% MaxIters - maximum number of iterations.
%
% OUTPUT
% ======
% bestX - array of best solution.
% bestFx - best function value.
%
% Example
% =======
%

  n = length(XLow);
  if length(XHi) < n, XHi = XHi(1) + zeros(1,n); end
  m = n + 1;

  bestX = XLow + (XHi - XLow) .* rand(1,n);
  bestFx = fx(bestX);
  for i=1:MaxMCiters
    X = XLow + (XHi - XLow) .* rand(1,n);
    Fx = fx(X);
    if Fx < bestFx
      bestX = X;
      bestFx = Fx;
      fprintf('%f ', X);
      fprintf('%20.18e at iter %i\n', bestFx, i);
      
      for j=1:MaxHCiters
         StepFactor = exp(log(InitStep) + log(FinStep/InitStep)*log(j)/log(MaxHCiters));
         
        for k=1:n
          X(k) = normrand(bestX(k), (XHi(k) - XLow(k)) * rand * StepFactor); 
        end
        Fx = fx(X);
        
        if Fx < bestFx
          bestX = X;
          bestFx = Fx;
          fprintf('%f ', X);
          fprintf('%20.18e at iter %i and iter2 %i\n', bestFx, i, j); 
        end
      end
    end
  end
end

function y = normrand(mean,sigma)
  y = mean + sigma * randn(1,1);
end